package Moo::Google::AuthStorage::DBI;
$Moo::Google::AuthStorage::DBI::VERSION = '0.03';

# ABSTRACT: Specific methods to fetch tokens from DBI data source (not written yet)

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Moo::Google::AuthStorage::DBI - Specific methods to fetch tokens from DBI data source (not written yet)

=head1 VERSION

version 0.03

=head1 AUTHOR

Pavel Serikov <pavelsr@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2017 by Pavel Serikov.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
